@@ -1,6 +1,6 @@
 /****************************************************************************
 **
-** Copyright (C) 2011 Nokia Corporation and/or its subsidiary(-ies).
+** Copyright (C) 2012 Nokia Corporation and/or its subsidiary(-ies).
 ** All rights reserved.
 ** Contact: Nokia Corporation (qt-info@nokia.com)
 **
@@ -650,7 +650,11 @@ void SymbianSbsv2MakefileGenerator::writeBldInfExtensionRulesPart(QTextStream& t
         foreach(QString targetItem, project->values(QLatin1String("QMAKE_INTERNAL_ET_PARSED_TARGETS.") + item)) {
             // Make sure targetpath is absolute
             QString absoluteTarget = QDir::cleanPath(outputDir.absoluteFilePath(targetItem));
+#if defined(Q_OS_WIN)
+            if (allPreDeps.contains(absoluteTarget, Qt::CaseInsensitive)) {
+#else
             if (allPreDeps.contains(absoluteTarget)) {
+#endif
                 QStringList deps = project->values(QLatin1String("QMAKE_INTERNAL_ET_PARSED_DEPS.") + item + targetItem);
                 QString commandItem =  project->values(QLatin1String("QMAKE_INTERNAL_ET_PARSED_CMD.") + item + targetItem).join(" ");
 
@@ -683,13 +687,24 @@ void SymbianSbsv2MakefileGenerator::writeBldInfExtensionRulesPart(QTextStream& t
     t << endl;
 
     // Write deployment rules
-    QString remoteTestPath = qt_epocRoot() + QLatin1String("epoc32/winscw/c/private/") + privateDirUid;
+    QString remoteTestPath = qt_epocRoot()
+        + QLatin1String("epoc32/release/winscw/udeb/z/private/") + privateDirUid;
     DeploymentList depList;
 
     //write emulator deployment
+    // There are deployment targets for both uded and urel emulators.
     t << "#if defined(WINSCW)" << endl;
     initProjectDeploySymbian(project, depList, remoteTestPath, false, true,
-        QLatin1String(EMULATOR_DEPLOYMENT_PLATFORM), QString(), generatedDirs, generatedFiles);
+        QLatin1String(EMULATOR_DEPLOYMENT_PLATFORM), QLatin1String(BUILD_DEBUG),
+        generatedDirs, generatedFiles);
+    writeSbsDeploymentList(depList, t);
+
+    depList.clear();
+    remoteTestPath = qt_epocRoot()
+        + QLatin1String("epoc32/release/winscw/urel/z/private/") + privateDirUid;
+    initProjectDeploySymbian(project, depList, remoteTestPath, false, true,
+        QLatin1String(EMULATOR_DEPLOYMENT_PLATFORM), QLatin1String(BUILD_RELEASE),
+        generatedDirs, generatedFiles);
     writeSbsDeploymentList(depList, t);
     t << "#endif" << endl;
 
