@@ -1,6 +1,6 @@
 /****************************************************************************
 **
-** Copyright (C) 2011 Nokia Corporation and/or its subsidiary(-ies).
+** Copyright (C) 2012 Nokia Corporation and/or its subsidiary(-ies).
 ** All rights reserved.
 ** Contact: Nokia Corporation (qt-info@nokia.com)
 **
@@ -167,8 +167,12 @@ Q_GLOBAL_STATIC(QThreadStorage<CCnvCharacterSetConverter *>,gs_converterStore);
 CCnvCharacterSetConverter *QSymbianTextCodec::converter()
 {
     CCnvCharacterSetConverter *&conv = gs_converterStore()->localData();
-    if (!conv)
+    if (!conv) {
+        QScopedPointer<CTrapCleanup> trap;
+        if (!User::TrapHandler())
+            trap.reset(CTrapCleanup::New());
         QT_TRAP_THROWING(conv = CCnvCharacterSetConverter::NewL())
+    }
     return conv;
 }
 
