@@ -1,6 +1,6 @@
 /****************************************************************************
 **
-** Copyright (C) 2011 Nokia Corporation and/or its subsidiary(-ies).
+** Copyright (C) 2012 Nokia Corporation and/or its subsidiary(-ies).
 ** All rights reserved.
 ** Contact: Nokia Corporation (qt-info@nokia.com)
 **
@@ -367,7 +367,7 @@ void QWSKeyboardHandler::processKeyEvent(int unicode, int keycode, Qt::KeyboardM
     QString str;
     if (unicode != 0xffff)
         str = QString(unicode);
-    QWindowSystemInterface::handleKeyEvent(0, type, keycode, modifiers, str);
+    QWindowSystemInterface::handleKeyEvent(0, type, keycode, modifiers, str, autoRepeat);
 #endif
 }
 
