@@ -1,6 +1,6 @@
 /****************************************************************************
 **
-** Copyright (C) 2011 Nokia Corporation and/or its subsidiary(-ies).
+** Copyright (C) 2012 Nokia Corporation and/or its subsidiary(-ies).
 ** All rights reserved.
 ** Contact: Nokia Corporation (qt-info@nokia.com)
 **
@@ -102,6 +102,10 @@ QT_BEGIN_NAMESPACE
     any way, but for a linear layout, the order is essential. When writing your own
     layout subclass, you are free to choose the API that best suits your layout.
 
+    For adding layout items to the custom layout, the QGraphicsLayout provides
+    convenience function addChildLayoutItem(). The function will take care of
+    automatically reparenting graphics items, if needed.
+
     \section1 Activating the Layout
 
     When the layout's geometry changes, QGraphicsLayout immediately rearranges
