@@ -1,6 +1,6 @@
 /****************************************************************************
 **
-** Copyright (C) 2011 Nokia Corporation and/or its subsidiary(-ies).
+** Copyright (C) 2012 Nokia Corporation and/or its subsidiary(-ies).
 ** All rights reserved.
 ** Contact: Nokia Corporation (qt-info@nokia.com)
 **
@@ -450,8 +450,8 @@ void QNetworkSessionPrivateImpl::open()
         isOpen = false;
         isOpening = false;
         iError = QNetworkSession::UnknownSessionError;
-        emit QNetworkSessionPrivate::error(iError);
         closeHandles();
+        emit QNetworkSessionPrivate::error(iError);
         syncStateWithInterface();    
     }
 }
@@ -723,8 +723,8 @@ void QNetworkSessionPrivateImpl::Error(TInt aError)
         activeConfig = QNetworkConfiguration();
         serviceConfig = QNetworkConfiguration();
         iError = QNetworkSession::RoamingError;
-        emit QNetworkSessionPrivate::error(iError);
         closeHandles();
+        emit QNetworkSessionPrivate::error(iError);
         QT_TRY {
             syncStateWithInterface();
             // In some cases IAP is still in Connected state when
@@ -1050,7 +1050,7 @@ void QNetworkSessionPrivateImpl::ConnectionStartComplete(TInt statusCode)
     qDebug() << "QNS this : " << QString::number((uint)this) << " - "
             << "RConnection::Start completed with status code: " << statusCode;
 #endif
-    delete ipConnectionStarter;
+    // ConnectionStarter *ipConnectionStarter will delete itself at the end of RunL
     ipConnectionStarter = 0;
 
     switch (statusCode) {
@@ -1076,8 +1076,8 @@ void QNetworkSessionPrivateImpl::ConnectionStartComplete(TInt statusCode)
                 isOpen = false;
                 isOpening = false;
                 iError = QNetworkSession::UnknownSessionError;
-                QT_TRYCATCH_LEAVING(emit QNetworkSessionPrivate::error(iError));
                 closeHandles();
+                QT_TRYCATCH_LEAVING(emit QNetworkSessionPrivate::error(iError));
                 if (!newActiveConfig.isValid()) {
                     // No valid configuration, bail out.
                     // Status updates from QNCM won't be received correctly
@@ -1125,8 +1125,8 @@ void QNetworkSessionPrivateImpl::ConnectionStartComplete(TInt statusCode)
             activeConfig = QNetworkConfiguration();
             serviceConfig = QNetworkConfiguration();
             iError = QNetworkSession::InvalidConfigurationError;
-            QT_TRYCATCH_LEAVING(emit QNetworkSessionPrivate::error(iError));
             closeHandles();
+            QT_TRYCATCH_LEAVING(emit QNetworkSessionPrivate::error(iError));
             QT_TRYCATCH_LEAVING(syncStateWithInterface());
             break;
         case KErrCancel: // Connection attempt cancelled
@@ -1144,8 +1144,8 @@ void QNetworkSessionPrivateImpl::ConnectionStartComplete(TInt statusCode)
             } else {
                 iError = QNetworkSession::UnknownSessionError;
             }
-            QT_TRYCATCH_LEAVING(emit QNetworkSessionPrivate::error(iError));
             closeHandles();
+            QT_TRYCATCH_LEAVING(emit QNetworkSessionPrivate::error(iError));
             QT_TRYCATCH_LEAVING(syncStateWithInterface());
             break;
     }
@@ -1226,8 +1226,8 @@ bool QNetworkSessionPrivateImpl::newState(QNetworkSession::State newState, TUint
         activeConfig = QNetworkConfiguration();
         serviceConfig = QNetworkConfiguration();
         iError = QNetworkSession::SessionAbortedError;
-        emit QNetworkSessionPrivate::error(iError);
         closeHandles();
+        emit QNetworkSessionPrivate::error(iError);
         // Start handling IAP state change signals from QNetworkConfigurationManagerPrivate
         iHandleStateNotificationsFromManager = true;
         emitSessionClosed = true; // Emit SessionClosed after state change has been reported
@@ -1572,12 +1572,14 @@ void ConnectionStarter::Start(TConnPref &pref)
 void ConnectionStarter::RunL()
 {
     iOwner.ConnectionStartComplete(iStatus.Int());
-    //note owner deletes on callback
+    delete this;
 }
 
 TInt ConnectionStarter::RunError(TInt err)
 {
     qWarning() << "ConnectionStarter::RunError" << err;
+    // there must have been a leave from iOwner.ConnectionStartComplete, in which case "delete this" in RunL was missed.
+    delete this;
     return KErrNone;
 }
 
